/*
#
#   hcal version 1.0 - postscript hebrew calendar generator
#   copyright 1994,1995 - Steven Weintraub
#
#   This program is created and distribed by Steven Weintraub.  It
#   can be freely distributed without permission as long as this
#   head is retained in all files it appears and the LICENCE file
#   accompanies it.
#
*/

#include <stdio.h>
#include <curses.h>
#ifdef SEQUENT
#include <string.h>
#else
#include <strings.h>
#endif
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#ifdef AIX
#include <sys/access.h>
#endif
#include <sys/stat.h>
#include <sys/types.h>
#include "cur.h"
#include "version.h"

#define PROTOTYPES
#if defined(SUN)
#undef PROTOTYPES
#endif

#define NO  0
#define YES 1

#define MAX_STR 256

#define BOX_WIDTH 102

#define SHAHARIT 0
#define MINCHA   1
#define MARIV    2
#define ROSH     7
#define CHAG     8


#define SET_JDATE		(1 << 0)
#define SET_HOLIDAY		(1 << 1)
#define SET_CANDLES		(1 << 2)
#define SET_SHABBAT		(1 << 3)
#define SET_CHANUKAH		(1 << 4)
#define SET_OMER		(1 << 5)
#define SET_PRAYER_DEFAULT	(0 << 6) 
#define SET_PRAYER_ROSH		(1 << 6)
#define SET_PRAYER_SHABBAT	(2 << 6)
#define SET_PRAYER_CHAG		(3 << 6)
#define SET_PRAYER_MANUAL	(4 << 6)
#define SET_PRAYER_MASK		(7 << 6)

#define TOP_MARGIN        40
#define BOTTOM_MARGIN     40
#define RIGHT_MARGIN      39
#define LEFT_MARGIN       39

#define TIME_INCR_SIZE        11
#define TIME_INCR_SIGN_SIZE   1
#define TIME_ROUND_SIZE       6
#define TIME_ROUND_SIGN_SIZE  1
#define TIME_TYPE_SIZE        4
#define TIME_CANDLES_SIZE     2
#define TIME_LIMIT_EARLY_SIZE 11
#define TIME_LIMIT_LATE_SIZE  11

#define TIME_INCR_SHIFT       0
#define TIME_INCR_SIGN_SHIFT  (TIME_INCR_SHIFT + TIME_INCR_SIZE)
#define TIME_ROUND_SHIFT      (TIME_INCR_SIGN_SHIFT + TIME_INCR_SIGN_SIZE)
#define TIME_ROUND_SIGN_SHIFT (TIME_ROUND_SHIFT + TIME_ROUND_SIZE)
#define TIME_TYPE_SHIFT       (TIME_ROUND_SIGN_SHIFT + TIME_ROUND_SIGN_SIZE)
#define TIME_CANDLES_SHIFT    (TIME_TYPE_SHIFT + TIME_TYPE_SIZE)
#define TIME_LIMIT_EARLY_SHIFT 0
#define TIME_LIMIT_LATE_SHIFT  (TIME_LIMIT_EARLY_SHIFT + TIME_LIMIT_EARLY_SIZE)

#define TIME_INCR_MASK        ((1 << TIME_INCR_SIZE) - 1)
#define TIME_INCR_SIGN_MASK   (((1 << TIME_INCR_SIGN_SIZE) - 1) << TIME_INCR_SIGN_SHIFT)
#define TIME_ROUND_MASK       (((1 << TIME_ROUND_SIZE) - 1) << TIME_ROUND_SHIFT)
#define TIME_ROUND_SIGN_MASK  (((1 << TIME_ROUND_SIGN_SIZE) - 1) << TIME_ROUND_SIGN_SHIFT)
#define TIME_TYPE_MASK        (((1 << TIME_TYPE_SIZE) - 1) << TIME_TYPE_SHIFT)
#define TIME_CANDLES_MASK     (((1 << TIME_CANDLES_SIZE) - 1) << TIME_CANDLES_SHIFT)
#define TIME_LIMIT_EARLY_MASK (((1 << TIME_LIMIT_EARLY_SIZE) - 1) << TIME_LIMIT_EARLY_SHIFT)
#define TIME_LIMIT_LATE_MASK  (((1 << TIME_LIMIT_LATE_SIZE) - 1) << TIME_LIMIT_LATE_SHIFT)

#define TIME_TYPE_NORMAL             ( 0 << TIME_TYPE_SHIFT)
#define TIME_TYPE_MIDNIGHT           ( 1 << TIME_TYPE_SHIFT)
#define TIME_TYPE_SUNRISE            ( 2 << TIME_TYPE_SHIFT)
#define TIME_TYPE_ZMAN_TEFILA        ( 3 << TIME_TYPE_SHIFT)
#define TIME_TYPE_HALACHIC_MIDDAY    ( 4 << TIME_TYPE_SHIFT)
#define TIME_TYPE_HALACHIC_MIDNIGHT  ( 5 << TIME_TYPE_SHIFT)
#define TIME_TYPE_MINCHA_GADOLA      ( 6 << TIME_TYPE_SHIFT)
#define TIME_TYPE_MINCHA_KATANA      ( 7 << TIME_TYPE_SHIFT)
#define TIME_TYPE_PLUG_HAMINCHA      ( 8 << TIME_TYPE_SHIFT)
#define TIME_TYPE_CANDLELIGHTING     ( 9 << TIME_TYPE_SHIFT)
#define TIME_TYPE_SUNSET             (10 << TIME_TYPE_SHIFT)
#define TIME_TYPE_THREE_STARS        (11 << TIME_TYPE_SHIFT)
#define TIME_TYPE_ZAMIN_DAY          (12 << TIME_TYPE_SHIFT)
#define TIME_TYPE_ZAMIN_NIGHT        (13 << TIME_TYPE_SHIFT)
#define TIME_TYPE_NONE               (15 << TIME_TYPE_SHIFT)

#define TIME_CANDLES_NONE      (0 << TIME_CANDLES_SHIFT)
#define TIME_CANDLES           (1 << TIME_CANDLES_SHIFT)
#define TIME_CANDLES_EARLY     (TIME_CANDLES | TIME_TYPE_CANDLELIGHTING)
#define TIME_CANDLES_LATE      (TIME_CANDLES | TIME_TYPE_THREE_STARS)
#define TIME_CANDLES_HAVDALAH  ((2 << TIME_CANDLES_SHIFT) | TIME_TYPE_THREE_STARS)

#define TIME_FORMAT_ABREV      0
#define TIME_FORMAT_CLIPPED    1
#define TIME_FORMAT_SHORT      2
#define TIME_FORMAT_ACTUAL     3
#define TIME_FORMAT_FULL       4

#define TYPE_MINYANW		0x0020 /* save first 31 bits for day info */
#define TYPE_MINYAND		0x0040
#define TYPE_JDATE		0x0060
#define TYPE_SHAHARIT		0x0080
#define TYPE_MINCHA		0x00a0
#define TYPE_MARIV		0x00c0
#define TYPE_MITZVAH		0x00e0
#define TYPE_HOLIDAY		0x0100
#define TYPE_CANDLES		0x0120
#define TYPE_SHABBAT		0x0140
#define TYPE_CHANUKAH		0x0160
#define TYPE_OMER		0x0180
#define TYPE_INFO		0x01a0

#define TYPE_ADD		0x0400
#define TYPE_ADD_MITZVAH	0x0420
#define TYPE_EDIT_MINYAN	0x0440
#define TYPE_DELETE		0x0460
#define TYPE_SHOW_ALL		0x0480
#define TYPE_SHOW_SOME		0x04a0
#define TYPE_QUIT		0x04c0
#define TYPE_ADD_MISC		0x04e0

#define TYPE_MAIN_MONTH         1
#define TYPE_MAIN_EDIT          2
#define TYPE_MAIN_PRINT         3
#define TYPE_MAIN_DATA          4
#define TYPE_MAIN_SAVE          5
#define TYPE_MAIN_READ          6
#define TYPE_MAIN_QUIT          7

#define TYPE_MASK		0x07e0
#define TYPE_BACK		0x001f

#define TOKEN_EOCM              1
#define TOKEN_EOF               2
#define TOKEN_EOLN              3
#define TOKEN_MATCH             4
#define TOKEN_STRING            5
#define TOKEN_INT               6
#define TOKEN_TIME              7
#define TOKEN_JMONTH            8
#define TOKEN_MONTH             9
#define TOKEN_DOT              10
#define TOKEN_LATITUDE         11
#define TOKEN_LONGITUDE        12
#define TOKEN_SPACE            13 /* for time parsing */
#define TOKEN_CHAR             14 /* for time parsing */


#define DAY_OF_WEEK(day)        (((((day) - 1) % 7) + first_week_day) % 7)

#define BASE_YEAR 1760

struct time_info {
	long	word1;
	long	word2;
};

typedef struct time_info time_info_t;

struct shabbat {
	int			 autogenerated;
	char			*name;
	char			*torah;
	char			*prophet;
	char			*schedule;
};

typedef struct shabbat shabbat_t;

struct mitzvah {
	int		 place;
	int		 type;
	char		*who;
	char		*schedule;
};

typedef struct mitzvah mitzvah_t;

struct information {
	int			 bold;
	int			 items;
	int			 autogenerated;
	time_info_t		 time;
	char			*calendar;
	char			*schedule;
	struct information	*next;
	struct information	*last;
};

typedef struct information info_t;

struct day_data {
	char		*jmonth;
	int		 jday;
	int		 jyear;
	int		 holiday;
	time_info_t	 candles;
	shabbat_t	*shabbat;
	int		 chanukah;
	int		 omer;
	int		 autogenerated;
	int		 edited;
	time_info_t	 shaharit;
	time_info_t	 mincha;
	time_info_t	 mariv;
	mitzvah_t	*mitzvah;
	info_t		*info;
	info_t		*info_last;
	struct day_data *next;
	struct day_data *last;
};

typedef struct day_data day_data_t;

struct city_info
  {
  char			*name;
  int			 latdeg;
  int			 latmin;
  int			 latsec;
  int			 longdeg;
  int			 longmin;
  int			 longsec;
  int			 tzone;
  int			 dst_start_month;
  int			 dst_start_day;
  int			 dst_end_month;
  int			 dst_end_day;
  cur_slist_t		*slist_ptr;
  struct city_info	*next;
  struct city_info	*last;
  };

typedef struct city_info city_t;

struct file_info {
  FILE *file;
  char *name;
  int line;
  struct file_info *next;
  struct file_info *last;
};

typedef struct file_info file_info_t;

struct ptable
  {
  int	 state;
  int	 token;
  char	*match;
  int	 action;
  int	 next;
  int	 enext;
  int	 param;
  };

typedef struct ptable ptable_t;

extern int		 asking;
extern int		 changed;
extern city_t		*cities;
extern city_t		 cities_orig[];
extern city_t		*city;
extern int		 city_number;
extern cur_slist_t	*city_slist;
extern int		 changed_all;
extern int		 current_month;
extern int		 current_year;
extern day_data_t	 day[33];
extern int		 delete;
extern int		 dirty;
extern int		 draw_month;
extern file_info_t	*file_bottom;
extern file_info_t	*file_current;
extern int		 first_week_day;
extern int		 flag_a;
extern char		*flag_c;
extern int		 flag_C;
extern int		 flag_i;
extern int		 flag_I;
extern int		 flag_L;
extern char		*flag_o;
extern char		*flag_p;
extern int		 flag_q;
extern char		*flag_s;
extern int		 flag_S;
extern char		*flag_t;
extern int		 margin_bottom;
extern int		 margin_left;
extern int		 margin_right;
extern int		 margin_top;
extern cur_mlist_t	*menu_candles;
extern cur_mlist_t	*menu_city;
extern cur_mlist_t	*menu_general;
extern cur_mlist_t	*menu_jdate;
extern cur_mlist_t	*menu_minyan_add;
extern cur_mlist_t	*menu_minyan_edit;
extern cur_mlist_t	*menu_mitzvah;
extern cur_mlist_t	*menu_shabbat;
extern cur_mlist_t	*menu_time_regular;
extern cur_mlist_t	*menu_time_zamin;
extern cur_mlist_t	*menu_time_zamin_day;
extern char		*menu_list_candles[];
extern char		*menu_list_city[];
extern char		*menu_list_general[];
extern char		*menu_list_jdate[];
extern char		*menu_list_minyan_add[];
extern char		*menu_list_minyan_edit[];
extern char		*menu_list_mitzvah[];
extern char		*menu_list_shabbat[];
extern char		*menu_list_time_regular[];
extern char		*menu_list_time_zamin[];
extern char		*menu_list_time_zamin_day[];
extern time_info_t	 minyan[10][3];
extern char		*mitztypes[];
extern char		*month[];
extern cur_slist_t	*month_menu;
extern int		 month_length[];
extern int		 month_lengths[];
extern int		 numdays;
extern int		 read_buffer[MAX_STR];
extern int		*read_place;
extern int		*read_end;
extern int		*read_token_start;
extern int		 reading_calendar;
extern char		 save_file_name[MAX_STR];
extern ptable_t	       **parse_entry;
extern ptable_t	       **parse_entry_time;
extern int		 parse_errors;
extern char		*parse_error_table[MAX_STR];
extern int		 parse_first;
extern int		 parse_have_month;
extern int		 parse_int_1;
extern int		 parse_int_2;
extern int		 parse_int_3;
extern int		 parse_string_level;
extern time_info_t	 parse_time_1;
extern time_info_t	 parse_time_2;
extern time_info_t	 parse_time_3;
extern int		 parse_int_5;
extern char		 parse_string_1[MAX_STR];
extern char		 parse_string_2[MAX_STR];
extern char		 parse_string_3[MAX_STR];
extern char		 parse_string_4[MAX_STR];
extern ptable_t		 parse_table[];
extern int		 parse_table_size;
extern ptable_t		 parse_table_time[];
extern int		 parse_table_time_size;
extern char		 print_file_name[MAX_STR];
extern char		*program;
extern char		*ps_back[];
extern char		*ps_front[];
extern char		*ps_middle[];
extern char		 schedule_file_name[MAX_STR];
extern int		 show_all;
extern char		 template_file_name[MAX_STR];
extern int		 tdirty;
extern char 		*time_types[];
extern cur_slist_t	*time_types_slist;
extern char		*weekdays[];
extern char		*weekth[];

#include "modules.h"
